//Statistical catch-at-age model
//Fishery selectivity estimated as free parameters
//Developed for ADMB Short Course 1, June 2006
//Created 4-27-06 by B. Linton
//Last modified 6-5-06 by B. Linton

DATA_SECTION
//Read in data from "catch.dat" file
  init_int fyear  //first year in model
  init_int lyear  //last year in model
  init_int fage  //first age in model
  init_int lage  //last age in model
  init_number M  //natural mortality
  init_vector total_catch_obs(fyear,lyear)  //observed total annual fishery catch
  init_matrix prop_CAA_obs(fyear,lyear,fage,lage)  //observed proportion of catch-at-age
  init_number sample_size  //number of fish sample for fishery age composition 
  init_vector effort(fyear,lyear)  //fishery effort
  init_number lambda  //relative variances of total catch to effort deviations (log scale)

//Define looping variables
  int i  //looping variable for years
  int j  //looping variable for ages

//Check that data read in correctly
//!!cout << lambda << endl;
//!!exit(5);

INITIALIZATION_SECTION
//Set parameter starting values
  log_recruit 10
  log_q -1
  log_log_total_catch_sd -1.5

PARAMETER_SECTION
//Phase 1 parameters
  init_number log_recruit(1)  //average recruitment (log scale)
  init_number log_q(1)  //catchabiltiy (log scale)

//Phase 2 parameters
  //Elements of bounded_dev_vector must be between lower and upper bound, and must sum to 0
  init_bounded_dev_vector log_recruit_devs(fyear,lyear,-50,50,2)  //deviations in annual recruitment (log scale)
  init_vector log_initpop_devs(fage,lage-1,2)  //deviations in abundance at age in first year (log scale)
  init_vector log_effort_devs(fyear,lyear,2)  //deviations in fishery effort (log scale)

//Phase 3 parameters
  init_vector log_select(fage,lage-1,3)  //fishery selectivity (log scale) for all ages, except age-5
  init_number log_log_total_catch_sd(3)  //standard deviation of log scale total catch (log scale)

//Calculated quantities
  matrix N(fyear,lyear,fage,lage)  //numbers of fish at age
  vector select(fage,lage)  //fishery selectivity all ages
  number q  //catchability
  matrix F(fyear,lyear,fage,lage)  //fishing mortaltiy
  matrix Z(fyear,lyear,fage,lage)  //total mortality
  matrix S(fyear,lyear,fage,lage)  //survival
  matrix CAA(fyear,lyear,fage,lage)  //catch-at-age (numbers of fish)
  vector total_catch_pred(fyear,lyear)  //predicted total annual catch (numbers of fish)
  matrix prop_CAA_pred(fyear,lyear,fage,lage)  //predicted proportion of catch-at-age
  number log_total_catch_sd  //standard deviation of total fishery catch (log scale)
  number log_effort_sd  //standard deviation of fishery effort (log scale)
  number n_catch  //number of total catch observations
  number n_effort  //number of effort deviations

//Define objective function value to be minimized  
  objective_function_value neg_log_like  //negative log likelihood 

PRELIMINARY_CALCS_SECTION
//Calculations that only need to be done once

//Calculate number of observations for negative log likelihood
  n_catch=size_count(total_catch_pred);  //for use in total catch component
  n_effort=size_count(log_effort_devs);  //for use in effort deviations component

PROCEDURE_SECTION
  get_mortality();  //Call function "get_mortality"
  get_numbers();  //Call function "get_numbers"
  get_catch();  //Call function "get_catch"
  evaluate_objective_function();  //Call function "evaluate_objective_function"

FUNCTION get_mortality
//This function estimates mortality and survival rates

//Single loop estimates selectivity
  for (j=fage;j<=lage;j++)
  {
    //Conditional statements fill "select" vector
    if (j<=4)
    {
      select(j)=exp(log_select(j));
    }
    if (j==5)
    {
      //Standardize selectivity by setting age-5 selectivity to 1.0
      select(j)=1.0;
    }
    if (j>=6)
    {
      select(j)=exp(log_select(j-1));
    }
  }

//Estimate fishing mortality
  q=exp(log_q);  //backtransform log scale catchability 
  for (i=fyear;i<=lyear;i++)
  {
    //Use separable fishing mortality
    //age component=select, year component=q*effort
    F(i)=select*q*effort(i);

    //Conditional statement includes error when log_effort_devs is active
    //Function active(x) returns value of TRUE when parameter x is being estimated
    if(active(log_effort_devs))
    {
      F(i)*=exp(log_effort_devs(i));
    }
  }

//Calculate total mortaltity    
  Z=F+M;
//Calculate survival
  S=exp(-1.0*Z); 

FUNCTION get_numbers
///This function estimates numbers of fish at age

//Loops set initial population values
  for (i=fyear;i<=lyear;i++)
  {
    //Number of recruits each year
    //Modeled as deviations around an average recruitment
    N(i,1)=exp(log_recruit+log_recruit_devs(i));
  }
  for (j=fage;j<=lage-1;j++)
  {
    //Numbers at age in initial year
    //Modeled as differences
    N(1,j+1)=exp(log(N(1,j))+log_initpop_devs(j));
  }

//Calculate remaining numbers at age
  for (i=fyear;i<lyear;i++)
  {
    for (j=fage;j<lage-1;j++)
    {
      N(i+1,j+1)=N(i,j)*S(i,j);
    }
    //Calculate numbers in last age as a plus group (age-8 and older)
    N(i+1,lage)=N(i,lage-1)*S(i,lage-1)+N(i,lage)*S(i,lage);
  }

FUNCTION get_catch
//This function estimates total annual catch and proportion of catch-at-age

//Estimate catch-at-age using Baranov's catch equation
  //Using ADMB functions
  //Function elem_prod(x,y)=first element of x * first element of y, ...,last element of x * last element of y
  //Function elem_div(x,y)=first element of x / first element of y, ...,last element of x / last element of y  
  CAA=elem_prod(elem_div(F,Z),elem_prod(1.0-S,N));

  //Using loops
  /*for (i=fyear;i<=lyear;i++)
  {
    for (j=fage;j<=lage;j++)
    {
      CAA(i,j)=F(i,j)/Z(i,j)*(1.0-S(i,j))*N(i,j);
    }
  }*/

//Loop calculates proportion of catch-at-age
  for (i=1;i<=lyear;i++)
  {
    //Function sum(x)=sum of elements of x
    total_catch_pred(i)=sum(CAA(i));  //calculate total annual catch
    prop_CAA_pred(i)=CAA(i)/total_catch_pred(i);
  }

FUNCTION evaluate_objective_function
//This function defines objective function value to be minimized

//Backtransform log scale parameter
  log_total_catch_sd=exp(log_log_total_catch_sd);  
//Calculate standard deviation of effort deviations (log scale) 
  log_effort_sd=log_total_catch_sd/sqrt(lambda);  

//Lognormal total annual catch component (ignoring constants)
  neg_log_like=0.5*n_catch*log(square(log_total_catch_sd))+0.5/square(log_total_catch_sd)*norm2(log(total_catch_obs)-log(total_catch_pred));

//Multinomial proportion of catch-at-age component (ignoring constants)
  neg_log_like+=-double(sample_size)*sum(elem_prod(prop_CAA_obs,log(prop_CAA_pred)));

//Lognormal effort deviations component (ignoring some constants)
  neg_log_like+=0.5*n_effort*log(square(log_effort_sd))+0.5/square(log_effort_sd)*norm2(log_effort_devs);

REPORT_SECTION
//Read out data to "catch.rep" file
  report << "POPULATION QUANTITIES" << endl;
  report << "Population abundance (numbers of fish), years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << N << endl;
  report << "Fishery selectivity, ages " << fage << "-" << lage << endl;
  report << select << endl;
  report << "Catchability" << endl;
  report << q << endl;
  report << "Fishing mortality, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << F << endl;
  report << "Natural mortality" << endl;
  report << M << endl;
  report << "Total mortality, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << Z << endl;
  report << "Standard deviation of total catch (log scale)" << endl;
  report << log_total_catch_sd << endl;
  report << "Standard deviation of effort deviations (log scale)" << endl;
  report << log_effort_sd << endl;
  report << " " << endl;
  report << "OBSERVED VS. PREDICTED QUANTITIES" << endl;
  report << "Observed total annual catch (numbers of fish), years " << fyear << "-" << lyear << endl;
  report << total_catch_obs << endl;
  report << "Predicted total annual catch (numbers of fish), years " << fyear << "-" << lyear << endl;
  report << total_catch_pred << endl;
  report << "Observed proportion of catch-at-age, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << prop_CAA_obs << endl;
  report << "Predicted proportion of catch-at-age, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << prop_CAA_pred << endl;
  report << "Observed effort (millions of ft of net), years " << fyear << "-" << lyear << endl;
  report << effort << endl;
  report << "Predicted effort (millions of ft. of gill net), years " << fyear << "-" << lyear << endl;
  report << elem_prod(effort,exp(log_effort_devs)) << endl; 





