//Simulation program for von B growth model
//With time-varying asymptotic length
//Developed for ADMB Short Course 2, June 2006
// Modified during course by JRB from datasim
//Created 5-23-06 by B. Linton
//Last modified 6-10-06 by B. Linton

DATA_SECTION
//Read in data from "fullsim.dat"
  init_int reps // number of replicates
  init_int plevels // number of process error levels
  init_int obslevels // number of obs error levels
//!! cout << reps << " " << plevels << " " << obslevels << endl;
  init_int fage  //first age
  init_int lage  //last age
  init_int fyear  //first year
  init_int lyear  //last year
  init_number in_Linf  //initial asymptotic length
  init_number K  //Brody growth coefficient
  init_number length_at_fage  //mean length at first age
  init_vector sd_log_lengthv(1,obslevels)  //standard deviation of mean length-at-age (log scale)
  init_vector sd_log_Linfv(1,plevels)  //standard deviation of asymptotic length (log scale) 
  init_number seed  //random number seed
  init_number sd_log_length
  init_number sd_log_Linf
//Data to be generated
  vector log_Linf_devs(fyear,lyear-1)  //deviations in Linf (log scale)
  vector Linf(fyear,lyear)  //time-varying Linf
  matrix length(fyear,lyear,fage,lage)  //true mean length at age
  matrix log_length_devs(fyear,lyear,fage,lage)  //deviations in true mean length at age (log scale)
  matrix obs_length(fyear,lyear,fage,lage)  //observed mean length at age

//Define looping variables in the data section, so you only have to define them once.
//Because they are in the data section they are valid everywhere in the model.
  int i  //looping variable for year
  int j  //looping variable for age
  int oe // current observation error level
  int pe // current process error level
  int rep // current replicate
//This code makes sure data reads in correctly
//!!cout << seed << endl;
//!!exit(5);
  
PARAMETER_SECTION
//Need at least one active parameter for model to run
  init_number dummy
//Need to declare an objective function value for model to compile
  objective_function_value fake 

PRELIMINARY_CALCS_SECTION
  system ("del parameter.rep"); 
  
  for(pe=1;pe<=plevels;pe++)
  {
  sd_log_Linf=sd_log_Linfv(pe);
  for(oe=1;oe<=obslevels;oe++)
  {
  sd_log_length=sd_log_lengthv(oe);
  for (rep=1;rep<=reps;rep++)
  {
//  cout << rep <<" " <<pe << " " << oe << endl;

  get_random_numbers();  //Call function "get_random_numbers"
  get_lengths();  //Call function "get_lengths"
  get_dat_file();  //Call function "get_dat_file"

  //Run .exe
  system ("growthtimeD -nox");

  } // end rep loop
  } // end oe loop
  } // end pe loop
  cout<< "all done" << endl;
  exit(0);

PROCEDURE_SECTION

//Define functions called above
FUNCTION get_random_numbers
//Generate random deviations with random number generator
  random_number_generator rng(seed);  //initialize random number generator
  log_Linf_devs.fill_randn(rng);  //fill vector with random normal deviations
  log_length_devs.fill_randn(rng);  //fill matrix with random normal deviations
  seed=seed+3;

FUNCTION get_lengths
//Generate asymptotic lengths
  Linf(fyear)=in_Linf;  //set asymptotic length in first year
  //Single loop generates asymptotic lengths with random walk
  for(i=fyear;i<=lyear-1;i++)
  {
    Linf(i+1)=exp(log(Linf(i))+log_Linf_devs(i)*sd_log_length); 
  }

//Single loop generates mean length-at-age in first year
  for(j=fage;j<=lage-1;j++)
  {
    //von Bertalanffy growth model
    //L(a+1) = L(a)+(Linf-L(a))*(1-exp(-K))
    length(fyear,j+1)=length(fyear,j)+(Linf(fyear)-length(fyear,j))*(1-exp(-K));
  }

//Single loop fills in mean length at first age
  for(i=fyear;i<=lyear;i++)
  {
    length(i,fage)=length_at_fage; 
  }  

//Double loop fills in remaining mean length-at-age
  for(i=fyear;i<lyear;i++)
  { 
    for(j=fage;j<lage;j++)
    {

      //von Bertalanffy growth model
      //L(t+1,a+1) = L(t,a)+(Linf(t)-L(t,a))*(1-exp(-K))
      length(i+1,j+1)=length(i,j)+(Linf(i+1)-length(i,j))*(1-exp(-K));
    }
  }

//Incorporate multiplicative observation error
  obs_length=elem_prod(length,exp(log_length_devs*sd_log_length)); 

FUNCTION get_dat_file
//Read out data to "growthtimeD.dat" file
  ofstream ofs("growthtimeD.dat");  //"ofs" command will now read out to "growthtimeD.dat" file
  ofs << "#replicate"<<endl;
  ofs << " "<< rep << endl;
  ofs << "#process error level"<<endl;
  ofs << " " << pe << endl;
  ofs << "#obs error level" <<endl;
  ofs << " " << oe << endl;
  ofs << "#Data for von Bertalanffy growth model with time-varying Linf" << endl;
  ofs << "#Data generated with datasim.tpl" << endl;
  ofs << " " << endl;
  ofs << "#Observed data" << endl;
  ofs << "#First and last year" << endl;
  ofs << " " << fyear << " " << lyear << endl;
  ofs << "#First and last age" << endl;
  ofs << " " << fage << " " << lage << endl;  
  ofs << "#Observed mean length at age(cm), years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  ofs << obs_length << endl;
  ofs << "#Ratio of relative variances of mean length-at-age to asymptotic length" << endl;
  ofs << " " << square(sd_log_length/sd_log_Linf) << endl;
  ofs << " " << endl;
  ofs << "#Parameter starting values" << endl;
  ofs << "#Initial asymptotic length (log scale)" << endl;
  ofs << " " << log(in_Linf) << endl;
  ofs << "#Brody growth coefficient (log scale)" << endl;
  ofs << " " << log(K) << endl;
  ofs << "#Mean length at first age (log scale)" << endl;
  ofs << " " << log(length_at_fage) << endl;  
  
