//Statistical catch-at-age model with double logistic function for selectivity
//For use with catchsim.tpl
//Developed for ADMB Short Course 2, June 2006
//Created 4-27-06 by B. Linton
//Last modified 6-21-06 by B. Linton

DATA_SECTION
//Read in data from "catchDL.dat" file
  //Identification numbers
  init_int nerror  //total catch SD level
  init_int nrep  //replicate number

  //Observed data
  init_int fyear  //first year in model
  init_int lyear  //last year in model
  init_int fage  //first age in model
  init_int lage  //last age in model
  init_number M  //natural mortality
  init_vector total_catch_obs(fyear,lyear)  //observed total annual fishery catch
  init_matrix prop_CAA_obs(fyear,lyear,fage,lage)  //observed proportion of catch-at-age
  init_number sample_size  //number of fish sample for fishery age composition 
  init_vector effort(fyear,lyear)  //fishery effort
  init_number lambda  //relative variances of total catch to effort deviations (log scale)

  //Parameter starting values
  init_number start_log_recruit  //for average recruitment
  init_number start_log_q  //for catchability
  init_number start_log_log_total_catch_sd  //for sd of log total catch

  init_vector test(1,3)  //data entry test values

//Define looping variables
  int i  //looping variable for years
  int j  //looping variable for ages

//Check that data read in correctly
//!!cout << test << endl;
//!!exit(5);

INITIALIZATION_SECTION
  //Selectivity parameters for double logistic
  log_inflection_1 1.39
  log_slope_1 0.34
  log_inflection_2 1.25
  log_slope_2 -0.69

PARAMETER_SECTION
//Phase 1 parameters
  init_number log_recruit(1)  //average recruitment (log scale)
  init_number log_q(1)  //catchabiltiy (log scale)

//Phase 2 parameters
  //Elements of bounded_dev_vector must be between lower and upper bound, and must sum to 0
  init_bounded_dev_vector log_recruit_devs(fyear,lyear,-50,50,2)  //deviations in annual recruitment (log scale)
  init_vector log_initpop_devs(fage,lage-1,2)  //deviations in abundance at age in first year (log scale)
  init_vector log_effort_devs(fyear,lyear,2)  //deviations in fishery effort (log scale)

//Phase 3 parameters
  init_number log_log_total_catch_sd(3)  //standard deviation of log scale total catch (log scale)
  //Selectivity parameters for double logistic function
  init_number log_inflection_1(3)  //first inflection point of curve (log scale)
  init_number log_slope_1(3)  //first slope of curve (log scale)
  init_number log_inflection_2(3)  //second inflection point of curve (log scale)
  init_number log_slope_2(3)  //second slope of curve (log scale)

//Calculated quantities
  matrix N(fyear,lyear,fage,lage)  //numbers of fish at age
  number inflection_1  //first inflection point of curve
  number slope_1  //first slope of curve
  number inflection_2  //second inflection point of curve
  number slope_2  //second slope of curve
  number select_denominator  //denominator for standardizing selectivity curve
  vector select_numerator(fage,lage)  //nonstandardized selectivity curve
  vector select(fage,lage)  //fishery selectivity all ages
  number q  //catchability
  matrix F(fyear,lyear,fage,lage)  //fishing mortaltiy
  matrix Z(fyear,lyear,fage,lage)  //total mortality
  matrix S(fyear,lyear,fage,lage)  //survival
  matrix CAA(fyear,lyear,fage,lage)  //catch-at-age (numbers of fish)
  vector total_catch_pred(fyear,lyear)  //predicted total annual catch (numbers of fish)
  matrix prop_CAA_pred(fyear,lyear,fage,lage)  //predicted proportion of catch-at-age
  number log_total_catch_sd  //standard deviation of total fishery catch (log scale)
  number log_effort_sd  //standard deviation of fishery effort (log scale)
  number n_catch  //number of total catch observations
  number n_effort  //number of effort deviations

//Define objective function value to be minimized  
  objective_function_value neg_log_like  //negative log likelihood 

PRELIMINARY_CALCS_SECTION
//Calculations that only need to be done once

//Set parameter starting values
  log_recruit=start_log_recruit;  //for average recruitment
  log_q=start_log_q;  //for catchability
  log_log_total_catch_sd=start_log_log_total_catch_sd;  //for sd of log total catch

//Calculate number of observations for negative log likelihood
  n_catch=size_count(total_catch_pred);  //for use in total catch component
  n_effort=size_count(log_effort_devs);  //for use in effort deviations component

PROCEDURE_SECTION
  get_mortality();  //Call function "get_mortality"
  get_numbers();  //Call function "get_numbers"
  get_catch();  //Call function "get_catch"
  evaluate_objective_function();  //Call function "evaluate_objective_function"

FUNCTION get_mortality
//This function estimates mortality and survival rates

//Calculate Selectivity
  //Backtransform log scale selectivity parameters
  inflection_1=exp(log_inflection_1);
  slope_1=exp(log_slope_1);
  inflection_2=exp(log_inflection_2);
  slope_2=exp(log_slope_2);

  //Double logistic function
  select_denominator=1/(1+exp(-slope_1*(5-inflection_1)))*(1-1/(1+exp(-slope_2*(5-inflection_2))));  //reference age set to 5
  for(j=fage;j<=lage;j++)
  {
    select_numerator(j)=1/(1+exp(-slope_1*(j-inflection_1)))*(1-1/(1+exp(-slope_2*(j-inflection_2))));
  }
  select=select_numerator/select_denominator;

//Estimate fishing mortality
  q=exp(log_q);  //backtransform log scale catchability 
  for (i=fyear;i<=lyear;i++)
  {
    //Use separable fishing mortality
    //age component=select, year component=q*effort
    F(i)=select*q*effort(i);

    //Conditional statement includes error when log_effort_devs is active
    //Function active(x) returns value of TRUE when parameter x is being estimated
    if(active(log_effort_devs))
    {
      F(i)*=exp(log_effort_devs(i));
    }
  }

//Calculate total mortaltity    
  Z=F+M;
//Calculate survival
  S=exp(-1.0*Z); 

FUNCTION get_numbers
///This function estimates numbers of fish at age

//Loops set initial population values
  for (i=fyear;i<=lyear;i++)
  {
    //Number of recruits each year
    //Modeled as deviations around an average recruitment
    N(i,1)=exp(log_recruit+log_recruit_devs(i));
  }
  for (j=fage;j<=lage-1;j++)
  {
    //Numbers at age in initial year
    //Modeled as difference from previous year
    N(1,j+1)=exp(log(N(1,j))+log_initpop_devs(j));
  }

//Calculate remaining numbers at age
  for (i=fyear;i<lyear;i++)
  {
    for (j=fage;j<lage-1;j++)
    {
      N(i+1,j+1)=N(i,j)*S(i,j);
    }
    //Calculate numbers in last age as a plus group (age-8 and older)
    N(i+1,lage)=N(i,lage-1)*S(i,lage-1)+N(i,lage)*S(i,lage);
  }

FUNCTION get_catch
//This function estimates total annual catch and proportion of catch-at-age

//Estimate catch-at-age using Baranov's catch equation
  //Using ADMB functions
  //Function elem_prod(x,y)=first element of x * first element of y, ...,last element of x * last element of y
  //Function elem_div(x,y)=first element of x / first element of y, ...,last element of x / last element of y  
  CAA=elem_prod(elem_div(F,Z),elem_prod(1.0-S,N));

  //Using loops
  /*for (i=fyear;i<=lyear;i++)
  {
    for (j=fage;j<=lage;j++)
    {
      CAA(i,j)=F(i,j)/Z(i,j)*(1.0-S(i,j))*N(i,j);
    }
  }*/

//Loop calculates proportion of catch-at-age
  for (i=1;i<=lyear;i++)
  {
    //Function sum(x)=sum of elements of x
    total_catch_pred(i)=sum(CAA(i));  //calculate total annual catch
    prop_CAA_pred(i)=CAA(i)/total_catch_pred(i);
  }

FUNCTION evaluate_objective_function
//This function defines objective function value to be minimized

//Backtransform log scale parameter
  log_total_catch_sd=exp(log_log_total_catch_sd);  
//Calculate standard deviation of effort deviations (log scale) 
  log_effort_sd=log_total_catch_sd/sqrt(lambda);  

//Lognormal total annual catch component (ignoring constants)
  neg_log_like=0.5*n_catch*log(square(log_total_catch_sd))+0.5/square(log_total_catch_sd)*norm2(log(total_catch_obs)-log(total_catch_pred));

//Multinomial proportion of catch-at-age component (ignoring constants)
  neg_log_like+=-double(sample_size)*sum(elem_prod(prop_CAA_obs,log(prop_CAA_pred)));

//Lognormal effort deviations component (ignoring some constants)
  neg_log_like+=0.5*n_effort*log(square(log_effort_sd))+0.5/square(log_effort_sd)*norm2(log_effort_devs);

  if(sd_phase())
  {
    //Output parameter estimates to parameter.rep file
    ofstream ofs("parameter.rep",ios::app);
    ofs << nerror << " " << nrep << " " << log_recruit << " " << log_q << " " << log_recruit_devs << " " << log_initpop_devs << " " << log_effort_devs << " " << log_log_total_catch_sd << " " << log_inflection_1 << " " << log_slope_1 << " " << log_inflection_2 << " " << log_slope_2 << " " << neg_log_like << " " << objective_function_value::gmax << endl;
  }

REPORT_SECTION
//Read out data to "catchDL.rep" file
  report << "POPULATION QUANTITIES" << endl;
  report << "Population abundance (numbers of fish), years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << N << endl;
  report << "Fishery selectivity, ages " << fage << "-" << lage << endl;
  report << select << endl;
  report << "Catchability" << endl;
  report << q << endl;
  report << "Fishing mortality, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << F << endl;
  report << "Natural mortality" << endl;
  report << M << endl;
  report << "Total mortality, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << Z << endl;
  report << "Standard deviation of total catch (log scale)" << endl;
  report << log_total_catch_sd << endl;
  report << "Standard deviation of effort deviations (log scale)" << endl;
  report << log_effort_sd << endl;
  report << " " << endl;
  report << "OBSERVED VS. PREDICTED QUANTITIES" << endl;
  report << "Observed total annual catch (numbers of fish), years " << fyear << "-" << lyear << endl;
  report << total_catch_obs << endl;
  report << "Predicted total annual catch (numbers of fish), years " << fyear << "-" << lyear << endl;
  report << total_catch_pred << endl;
  report << "Observed proportion of catch-at-age, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << prop_CAA_obs << endl;
  report << "Predicted proportion of catch-at-age, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  report << prop_CAA_pred << endl;
  report << "Observed effort (millions of ft of net), years " << fyear << "-" << lyear << endl;
  report << effort << endl;
  report << "Predicted effort (millions of ft. of gill net), years " << fyear << "-" << lyear << endl;
  report << elem_prod(effort,exp(log_effort_devs)) << endl; 
  




