//Simulation study using catch-at-age model with double logistic function for selectivity
//With low and high levels of total catch error
//Developed By B. Linton for ADMB Short Course 2, June 2006
//Last modified 6-21-06 by BCL

DATA_SECTION
//Read in data from catchsim.dat
  init_int fyear  //first year
  init_int lyear  //last year
  init_int fage  //first age
  init_int lage  //last age
  init_int nages_plus //number of ages to create plus group in first year
  init_number Nfyear_avg  //average recruitment for numbers at age in first year
  init_number log_Nfyear_sd  //sd of log recruitment for numbers at age in first year
  init_number alpha  //Ricker stock-recruitment parameter 
  init_number beta  //Ricker stock-recruitment parameter 
  init_number log_recruit_sd  //sd of log recruitment
  init_number M  //natural mortality
  init_vector effort(fyear,lyear)  //fishing effort 
  init_vector select(fage,lage)  //selectivity
  init_number q  //catchabitlity
  init_number log_effort_sd  //sd of log catchability
  init_number log_total_catch_sd_low  //low level sd of log total catch
  init_number log_total_catch_sd_hi  //high level sd of log total catch
  init_number Nsample  //effective sample size for catch age composition
  init_int nrep  //number of replicates
  init_int rnseed  //random number seed
  init_vector test(1,3)  //data entry test values

//Calculated quantities
  vector Nfyear(fage,nages_plus)  //recruitments for numbers at age in first year
  vector Nfyear_dev(fage,nages_plus)  //deviations in recruitments for numbers at age in first year
  vector Zfyear(fage,nages_plus)  //total mortality for numbers at age in first year
  vector qfyear_dev(fage,nages_plus-1)  //deviations in catchability in first year
  vector recruit_dev(fyear+1,lyear)  //deviations in recruitment
  vector spawn(fyear,lyear)  //spawning stock in each year
  matrix N(fyear,lyear,fage,lage)  //numbers at age
  vector Ntotal(fyear,lyear)  //total numbers for each year
  matrix Z(fyear,lyear,fage,lage)  //total mortality
  matrix S(fyear,lyear,fage,lage)  //survival
  matrix F(fyear,lyear,fage,lage)  //fishing mortality
  vector effort_dev(fyear,lyear)  //deviations in catchability
  matrix CAA(fyear,lyear,fage,lage)  //fishery catch at age
  vector total_catch(fyear,lyear)  //total annual catch
  matrix prop_CAA(fyear,lyear,fage,lage)  //proportion of catch at age
  vector total_catch_dev(fyear,lyear)  //deviations for measurement error in total catch
  number log_total_catch_sd  //sd of log scale total catch
  matrix prop_CAA_dev(fyear,lyear,1,Nsample)  //deviations for multinomial error in proportion of catch at age
  vector total_catch_obs(fyear,lyear)  //observed total catch
  matrix CAA_obs(fyear,lyear,fage,lage)  //observed catch at age
  matrix prop_CAA_obs(fyear,lyear,fage,lage)  //observed proportion of catch at age
  number lambda  //ratio of relative variances of total catch to effort
  number recruit_avg  //average recruitment starting value

//Declare looping variables
  int i  //loop over years
  int j  //loop over ages
  int k  //loop over replicates
  int l  //loop over levels of log total catch sd

//Check that data reads in correctly
//!!cout << test << endl;
//!!exit(5);

PARAMETER_SECTION
//Need one active parameter for model to run
  init_number dummy
//Need objective function value for model to compile
  objective_function_value fake

PRELIMINARY_CALCS_SECTION
//Delete old parameter.rep file from previous run
  system ("del parameter.rep");  

//Loop over levels of log total catch sd
  for (l=1;l<=2;l++)
  {
    //Assign value for sd of log total catch
    if (l==1)
    {
      log_total_catch_sd=log_total_catch_sd_low;
    }
    if (l==2)
    {
      log_total_catch_sd=log_total_catch_sd_hi;
    }

    //Loop over replicates
    for (k=1;k<=nrep;k++)
    {
      //Use functions to generate dat file
      get_random_numbers();
      get_mortality();
      get_numbers();
      get_catch();
      get_datafile();

      //Run catchDL.exe
      system ("catchDL -nox"); 

      //Identify different runs in Run-Time Message buffer
      cout << "________________________________________________________________________________" << endl;
      cout << "Log total catch sd level: " << l << "  " << "Replicate: " << k << endl;
      cout << "Random number seed: " << rnseed << endl;
      cout << "________________________________________________________________________________" << endl;
    }
  }
  cout << "***** THE END *****" << endl;
  exit(0);

PROCEDURE_SECTION
//Define functions used in preliminary calcs section

FUNCTION get_random_numbers
  random_number_generator rng(rnseed);  //initialize random number generator

  //Fill vectors with standard normal random numbers
  Nfyear_dev.fill_randn(rng);  //deviations in recruitments for numbers at age in first year
  qfyear_dev.fill_randn(rng);  //deviations in catchability in first year 
  recruit_dev.fill_randn(rng);  //deviations in recruitment 
  effort_dev.fill_randn(rng);  //deviations in catchability  
  //Fill matrix with uniform random numbers
  prop_CAA_dev.fill_randu(rng);  //proportions of catch-at-age

  //Change random number seed each time function called
  rnseed=rnseed+3;

FUNCTION get_mortality
//Calculate mortality rates

  //Set value for total mortality for numbers at age in first year
  Zfyear=0.0;
  //Double loop calculates total mortlaity for numbers at age in first year
  for (j=fage+1;j<=nages_plus;j++)
  {
    if (j<=lage)
    {
      Zfyear(j)=Zfyear(j-1)+select(j-1)*q*exp(qfyear_dev(j-1)*log_effort_sd)*effort(1)+M;
    }
    if (j>=lage+1)  
    {
      //Total mortality for ages in plus group
      Zfyear(j)=Zfyear(j-1)+select(lage)*q*exp(qfyear_dev(j-1)*log_effort_sd)*effort(1)+M;
    }
  }    

//Loop calculates fishing mortality
  for (i=fyear;i<=lyear;i++)
  {
    F(i)=select*q*effort(i)*exp(effort_dev(i)*log_effort_sd);  
  }

//Calculate total mortality
  Z=F+M;
//Calculate survival
  S=exp(-1.0*Z);

FUNCTION get_numbers
//Calculate recruitments for numbers at age in first year
  Nfyear=Nfyear_avg*exp(Nfyear_dev*log_Nfyear_sd);
 
//Loop fills in numbers in first year
  for (j=fage;j<=nages_plus;j++)
  {
    if (j<=lage)
    {
      N(1,j)=Nfyear(j)*exp(-1.0*Zfyear(j));  //numbers at age in first year
    }
    if (j>=lage+1)
    {
      N(1,lage)+=Nfyear(j)*exp(-1.0*Zfyear(j));  //numbers at age for plus group
    }
  }
  Ntotal(1)=sum(N(1));  //total numbers in first year
  spawn(1)=0.5*(Ntotal(1)-N(1,1)-N(1,2));  //spawning stock assumes knife-edge maturity at age 3 and 1:1 sex ratio

//Double loop fills in numbers in subsequent years
  for (i=fyear;i<=lyear-1;i++)
  {
    //Ricker stock-recruitment for numbers in first age
    N(i+1,1)=alpha*spawn(i)*exp(-1.0*beta*spawn(i))*exp(recruit_dev(i+1)*log_recruit_sd); 

    for (j=fage;j<=lage-2;j++)
    {
      N(i+1,j+1)=N(i,j)*S(i,j);  //fill in rest of numbers at age
    }

    N(i+1,lage)=N(i,lage)*S(i,lage)+N(i,lage-1)*S(i,lage-1);  //accounts for plus group in last age
    Ntotal(i+1)=sum(N(i+1));  //total numbers for year
    spawn(i+1)=0.5*(Ntotal(i+1)-N(i+1,1)-N(i+1,2));  //spawning stock assumes knife-edge maturity at age 3 and 1:1 sex ratio
  }

FUNCTION get_catch
//Calculate catch-at-age
  CAA=elem_prod(elem_div(F,Z),elem_prod(1.0-S,N));  //Baranov's catch equation

//Loop calculates total catch and proportion of catch-at-age
  for (i=fyear;i<=lyear;i++)
  {
    total_catch(i)=sum(CAA(i));  //total annual catch
    prop_CAA(i)=CAA(i)/total_catch(i);  //proportion of catch-at-age
    //See multinomial function defined below
    CAA_obs(i)=multinomial(Nsample, prop_CAA_dev(i), prop_CAA(i), lage);  //multinomial observed catch-at-age
    prop_CAA_obs(i)=CAA_obs(i)/sum(CAA_obs(i));  //observed proportion of catch-at-age
  }

//Calcualte observed total annual catch with lognormal error
  total_catch_obs=elem_prod(total_catch,exp(total_catch_dev*log_total_catch_sd));  

FUNCTION get_datafile
//Calculate ratio of relative variances
  lambda=square(log_total_catch_sd/log_effort_sd);
//Calculate average recruitment starting value
  recruit_avg=sum(column(N,1))/(lyear-fyear+1);

//Read out data to new dat file
  ofstream data("catchDL.dat");  //"data" command reads out to catchDL.dat
  data << "#Data for simulation study using catchDL.tpl" << endl;
  data << "#Total catch SD level and replicate number" << endl;
  data << " " << l << " " << k << endl;
  data << "#First and last year" << endl;
  data << " " << fyear << " " << lyear << endl;
  data << "#First and last age" << endl;
  data << " " << fage << " " << lage << endl;
  data << "#Natural mortality" << endl;
  data << " " << M << endl;
  data << "#Observed total annual fishery carch, years are row, ages columns" << endl;
  data << total_catch_obs << endl;
  data << "#Observed proportion of catch-at-age, years are row, ages columns" << endl;
  data << prop_CAA_obs << endl;
  data << "#Number of fish sampled each year for age composition" << endl;
  data << " " << Nsample << endl;
  data << "#Observed fishery effort (millions of ft of net)" << endl;
  data << " " << effort << endl;
  data << "#Ratio of relative variances of total catch to catchability deviations" << endl;
  data << " " << lambda << endl;
  data << "#Average recruitment starting value" << endl;
  data << " " << log(recruit_avg) << endl;
  data << "#Catchability starting value" << endl;
  data << " " << log(q) << endl;
  data << "#SD of log total catch starting value" << endl;
  data << " " << log(log_total_catch_sd) << endl;
  data << "#Data entry test values" << endl;
  data << test << endl; 

FUNCTION dvector multinomial(long N, dvector rand_uniform, dvector p, int dim_p)
//Function created by M. Wilberg to generate multinomial observed catch-at-age
  if (0.999999>sum(p)>1.000001) 
  {
    cout << "vector of proportions does not sum to one for multinomial random sample" << endl; 
    exit(1);
  } 
  //define local variables to use in this function
  dvector sample(1,dim_p);  //create vector for sample from multinomial
  //dvector sample_p(1,dim_p);
  double p1;
  double p2;
  int i;
  int j;
  sample.initialize();
  rand_uniform=sort(rand_uniform);  //sort random numbers
  p1=0;
  p2=p(1);
  j=1;
  for (i=1;i<=dim_p;i++)
  {
    //cout << "p1=" << p1 << " p2=" << p2 << endl;
    sample(i)=0;
    //assign random uniform numbers to proper bins for multinomial
    while (j <= N && rand_uniform(j) > p1 && rand_uniform(j) <= p2)
    {
      sample(i)++;
      j++;
    }
    if (i < dim_p)
    {
      p1=p2;
      p2+=p(i+1);
    }
  }
  return(sample);


  

  
