//von Bertalanffy growth model with time-varying asymptotic length
//For use with datasim.tpl
//Developed By B. Linton for ADMB Short Course 2, June 2006
//Based on code by M. Wilberg for yellow perch in Lake Michigan
//Last modified 6-10-06 by B. Linton

DATA_SECTION
//Read in data from "growthtimeD.dat"
  //Observed data
  init_int fyear  //first year of data
  init_int lyear  //last year of data
  init_int fage  //first age in model
  init_int lage  //last age in model
  init_matrix obs_length(fyear,lyear,fage,lage)  //observed mean length-at-age
  //ratio of relative variances for  mean length-at-age to annual asymptotic length deivations (log scale)
  init_number lambda

  //Parameter starting values
  init_number start_log_Linf  //for initial asymptotic length (log scale)
  init_number start_log_K  //for Brody growth coefficient (log scale)
  init_number start_log_length_at_fage  //for mean length at first age (log scale)

//Define looping variables in the data section, so you only have to define them once.
//Because they are in the data section they are valid everywhere in the model.
  int i;  //looping variable for year
  int j;  //looping variable for age

//Check that data reads in correctly
//!!cout << start_log_length_at_fage << endl;
//!!exit(5);

PARAMETER_SECTION
//Define parameters to be estimated in model
  init_number log_length_at_fage  //mean length at first age (log scale)
  init_number log_Linf  //initial asymptotic length (log scale)
  init_number log_K  //Brody growth coefficient (log scale)
  init_vector log_init_length(fage+1,lage-1)  //mean length-at-age in the first year (log scale), ages 2-9
  init_vector log_Linf_devs(fyear+1,lyear-1)  //annual deviations in asymptotic length (log scale)

//Define quantities to be calculated from parameters
  vector Linf(fyear+1,lyear)  //vector of time-varying asymptotic lengths
  matrix pred_length(fyear,lyear,fage,lage)  //matrix of predicted mean length-at-age
  number rss_length  //residual sums of squares for mean length-at-age
  number rss_Linf_devs  //residual sums of squares for asymptotic length deviations
  number sd_log_length  //standard deviation of mean length-at-age (log scale)
  number sd_log_Linf  //standard deviation of annual asymptotic length deivations (log scale)
  number n_length  //number of mean length-at-age observations
  number n_Linf_devs  //number of asymptotic length deviations
  
//Value for the objective function to minimize
  objective_function_value conc_like    //weighted negative log concentrated likelihood value

PRELIMINARY_CALCS_SECTION
//For calculations that are only made once

//Specify parameter starting values
  log_Linf=start_log_Linf;  //for initial asymptotic length (log scale)
  log_K=start_log_K;  //for Brody growth coefficient (log scale)
  log_length_at_fage=start_log_length_at_fage;  //for mean length at first age (log scale)

//Calculate number of observations for concentrated likelihood
  //function size_count(x) = number of elements in object x
  n_length=size_count(pred_length);  //for mean length-at-age component
  n_Linf_devs=size_count(log_Linf_devs);  //for asymptotic length deviations component

PROCEDURE_SECTION
  get_Linf();  //Call function "get_Linf"
  get_length_at_age();  //Call function "get_length_at_age"
  evaluate_objective_function();  //Call function "evaluate_the_objective_function"

//Define functions used above
FUNCTION get_Linf
//This function estimates time-varying asymptotic lengths

  Linf(fyear+1)=exp(log_Linf);  //backtransform log scale parameter
     
//Loop fills in annual asymptotic length values according to random walk
//In random walk, Linf in time t+1 = Linf in time t * random deviation
  for (i=fyear+1;i<=lyear-1;i++)
  {
    Linf(i+1)=exp(log(Linf(i))+log_Linf_devs(i));  
  }

FUNCTION get_length_at_age
//This function estimates mean length-at-age

//Loop fills in predicted mean length-at-age matrix
  for (i=fyear;i<=lyear;i++)
  {
    //Fill in first age mean lengths
    pred_length(i,fage)=exp(log_length_at_fage);
  }

//Loop fills in initial mean lengths for ages 2-9
  for (j=fage+1;j<=lage-1;j++)
  {
    //Fill in mean lengths for ages 2 to 9 in the first year
    pred_length(fyear,j)=exp(log_init_length(j));
  }     

//Fill in the last initial age (little data to fit model)  
  pred_length(fyear,lage)=exp(log_Linf); 

//Double loop to fill in predicted mean length-at-age matrix
  for (i=fyear;i<lyear;i++)
  {
    for (j=fage;j<lage;j++)
    {
      //von Bertalanffy growth model
      //L(t+1,a+1) = L(t,a)+(Linf(t)-L(t,a))*(1-exp(-K))
      pred_length(i+1,j+1)=pred_length(i,j)+(Linf(i+1)-pred_length(i,j))*(1-exp(-exp(log_K)));

      //Conditional statement makes it so fish can't shrink
      //Because of this, model is not full differentiable over all parameter values
      if (pred_length(i+1,j+1)<pred_length(i,j))
      {
        pred_length(i+1,j+1)=pred_length(i,j);
      }
    }
  }    

FUNCTION evaluate_objective_function
//This function defines objective function value to be minimized

//Calculate residual sums of squares
  //For mean length-at-age component
  rss_length=norm2(log(obs_length)-log(pred_length));

  //For asymptotic length deviations component  
  rss_Linf_devs=norm2(log_Linf_devs);  

//Minimize the weighted negative log concentrated likelihood
  conc_like=((n_length+n_Linf_devs)/2.0)*log(rss_length+lambda*rss_Linf_devs);
 

REPORT_SECTION
//Calculate standard deviations 
  sd_log_length=sqrt((rss_length+lambda*rss_Linf_devs)/(n_length+n_Linf_devs));  //for mean length-at-age
  sd_log_Linf=sd_log_length/sqrt(lambda);  //for asymptotic length

//Read out quantities to "growthtimeD.rep" file
  report << "Annual asymptotic lengths (Linf), years 2-10" << endl;
  report << Linf << endl;
  report << "Growth coefficient (K)" << endl;
  report << exp(log_K) << endl;
  report << "Observed mean length-at-age matrix, years 1-10, ages 1-10" << endl;
  report << obs_length << endl;
  report << "Predicted mean length-at-age matrix, years 1-10, ages 1-10" << endl;
  report << pred_length << endl;
  report << "Standard deviation of mean length-at-age (log scale)" << endl;
  report << sd_log_length << endl;  
  report << "Standard deviation of asymptotic length deviations (log scale)" << endl;
  report << sd_log_Linf << endl;     
  
