//Simulation study using von Bertalanffy growth model with time-varying asymptotic length
//With low and high levels of mean length-at-age error
//Developed By B. Linton for ADMB Short Course 2, June 2006
//Created 6-7-06 by B. Linton
//Last modified 6-21-06 by B. Linton

GLOBALS_SECTION
//Variables declared in global section are valid everywhere in model

//Declare looping variables 
  int i;  //looping variable for years
  int j;  //looping variable for ages
  int k;  //looping variable for replicates
  int l;  //looping variable for mean length-at-age error levels
  int m;  //looping variable for asymptotic length error levels

DATA_SECTION
//Read in data from control.dat file
  init_int nrep  //number of replicates for simulation study
  init_int nerr_length //number of levels of mean length-at-age error
  init_int nerr_Linf  //number of levels of asymptotic length error
  init_int fage  //first age
  init_int lage  //last age
  init_int fyear  //first year
  init_int lyear  //last year
  init_number in_Linf  //initial asymptotic length
  init_number K  //Brody growth coefficient
  init_number length_at_fage  //mean length at first age
  init_number sd_log_length_low  //low level standard deviation of mean length-at-age (log scale)
  init_number sd_log_length_hi  //high level standard deviation of mean length-at-age (log scale)
  init_number sd_log_Linf_low  //low level standard deviation of asymptotic length (log scale) 
  init_number sd_log_Linf_hi  //high level standard deviation of asymptotic length (log scale) 
  init_number seed  //random number seed

//Data to be generated
  vector log_Linf_devs(fyear,lyear)  //deviations in Linf (log scale)
  vector Linf(fyear,lyear)  //time-varying Linf
  matrix length(fyear,lyear,fage,lage)  //true mean length at age
  matrix log_length_devs(fyear,lyear,fage,lage)  //deviations in true mean length at age (log scale)
  matrix obs_length(fyear,lyear,fage,lage)  //observed mean length at age
  number sd_log_length  //standard deviation of mean length-at-age (log scale)
  number sd_log_Linf  //standard deviation of asymptotic (log scale)

//Check that data reads in correctly
//!!cout << seed << endl;
//!!exit(5);

PARAMETER_SECTION
//Need one active parameter for model to run
  init_number dummy
//Need objective function value for model to compile
  objective_function_value f

PRELIMINARY_CALCS_SECTION
//For calculations that only need to be done once

  //Delete old parameter.rep file from previous run
  system("del parameter.rep");  

  //Triple loop to run growthtimeF.exe with each dat file
  for(l=1;l<=nerr_length;l++)
  {
    //Specify level of mean length-at-age error
    if(l==1)
    {
      sd_log_length=sd_log_length_low;  //use low level of error
    }
    if(l==2)
    {
      sd_log_length=sd_log_length_hi;  //use high level of error
    }
    for(m=1;m<=nerr_Linf;m++)
    {
      //Specify level of mean length-at-age error
      if(m==1)
      {
        sd_log_Linf=sd_log_Linf_low;  //use low level of error
      }
      if(m==2)
      {
        sd_log_Linf=sd_log_Linf_hi;  //use high level of error
      }
      for(k=1;k<=nrep;k++)
      {
        
        //Use functions to create dat file
        get_random_numbers();  //Call function "get_random_numbers"
        get_lengths();  //Call function "get_lengths"
        get_dat_file();  //Call function "get_dat_file"

        //Run growthtimF.exe
        system("growthtimeF");

        //Help identify model runs in Run-Time Messages buffer
        cout << "_____________________________________________" << endl;
        cout << "Observation error level: " << l << "  Process error level: " << m << "   Replicate: " << k << endl;
        cout << "Random number seed: " << seed << endl;
        cout << "_____________________________________________" << endl;
      }
    }
  }
  cout << "**** THE END ****" << endl;  //lets you know that all model runs are completed
  exit(0);

PROCEDURE_SECTION
//Define functions called in preliminary calcs section
FUNCTION get_random_numbers
//Generate random deviations with random number generator
  random_number_generator rng(seed);  //initialize random number generator
  log_Linf_devs.fill_randn(rng);  //fill vector with random normal deviations
  log_length_devs.fill_randn(rng);  //fill matrix with random normal deviations

//Change seed each time function is called
  seed=seed+3.0;  

FUNCTION get_lengths
//Generate asymptotic lengths
  Linf(fyear)=in_Linf;  //set asymptotic length in first year
  //Single loop generates asymptotic lengths with random walk
  for(i=fyear;i<=lyear-1;i++)
  {
    Linf(i+1)=exp(log(Linf(i))+log_Linf_devs(i)*sd_log_Linf); 
  }

//Single loop generates mean length-at-age in first year
  for(j=fage;j<=lage-1;j++)
  {
    //von Bertalanffy growth model
    //L(a+1) = L(a)+(Linf-L(a))*(1-exp(-K))
    length(fyear,j+1)=length(fyear,j)+(Linf(fyear)-length(fyear,j))*(1-exp(-K));
  }

//Single loop fills in mean length at first age
  for(i=fyear;i<=lyear;i++)
  {
    length(i,fage)=length_at_fage; 
  }  

//Double loop fills in remaining mean length-at-age
  for(i=fyear;i<lyear;i++)
  { 
    for(j=fage;j<lage;j++)
    {

      //von Bertalanffy growth model
      //L(t+1,a+1) = L(t,a)+(Linf(t)-L(t,a))*(1-exp(-K))
      length(i+1,j+1)=length(i,j)+(Linf(i+1)-length(i,j))*(1-exp(-K));
    }
  }

//Incorporate multiplicative observation error
  obs_length=elem_prod(length,exp(log_length_devs*sd_log_length)); 

FUNCTION get_dat_file
//Read out data to "growthtimeF.dat" file
  ofstream ofs("growthtimeF.dat");  //"ofs" command will now read out to "growthtimeF.dat" file
  ofs << "#Data for von Bertalanffy growth model with time-varying Linf" << endl;
  ofs << "#Data generated with fullsim.tpl" << endl;
  ofs << " " << endl;
  ofs << "#Identification numbers" << endl;
  ofs << "#Mean length-at-age error level" << endl;
  ofs << " " << l << endl;
  ofs << "#Asymptotic length error level" << endl;
  ofs << " " << m << endl;
  ofs << "#Replicate number" << endl;
  ofs << " " << k << endl;
  ofs << " " << endl;
  ofs << "#Observed data" << endl;
  ofs << "#First and last year" << endl;
  ofs << " " << fyear << " " << lyear << endl;
  ofs << "#First and last age" << endl;
  ofs << " " << fage << " " << lage << endl;  
  ofs << "#Observed mean length at age(cm), years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  ofs << obs_length << endl;
  ofs << "#Ratio of relative variances of mean length-at-age to asymptotic length" << endl;
  ofs << " " << square(sd_log_length/sd_log_Linf) << endl;
  ofs << " " << endl;
  ofs << "#Parameter starting values" << endl;
  ofs << "#Initial asymptotic length (log scale)" << endl;
  ofs << " " << log(in_Linf) << endl;
  ofs << "#Brody growth coefficient (log scale)" << endl;
  ofs << " " << log(K) << endl;
  ofs << "#Mean length at first age (log scale)" << endl;
  ofs << " " << log(length_at_fage) << endl;  
